## es_qwen2 instructions

### 1. Install qwen2 deb

```shell
sudo apt install es-sdk-sample-npu-qwen

# notice: for deepseek based on release-2025-0530, we need to update qwen deb and dsp deb.
sudo dpkg -i es-sdk-sample-npu-qwen_1.7.0_riscv64.deb
sudo dpkg -i es-sdk-dsp_1.7.0_riscv64.deb
```

### 2. How to build?

```shell
# The qwen2 src is in /opt/eswin/sample-code/npu_sample/qwen_sample/src after installed qwen2 deb
cd ${qwen2_src}

# notice: ${cross_compile_prefix} can set to /usr/bin/ on the board or your CROSS_COMPILE path like this(/host/bin/riscv64-unknown-linux-gnu-) for cross compile.
export CROSS_COMPILE=${cross_compile_prefix}
./build.sh --sys_root=<sys_root_dir> --prefix=<install_prefix>
```

### 3. Model deployment

**dowload models from es-model-zoo**

```shell
cd es-model-zoo/
git lfs pull --include="llm/es-hw-models/qwen2/models/qwen2_0_5b_1k_int8/*" --exclude=""
git lfs pull --include="llm/es-hw-models/qwen2/models/qwen2_7b_1k_int8_peer/*" --exclude=""
```

**copy models to board**

```shell
cp -r ${qwen2_0_5b_1k_int8} /opt/eswin/sample-code/npu_sample/qwen_sample/models/
cp -r ${qwen2_7b_1k_int8_peer} /opt/eswin/sample-code/npu_sample/qwen_sample/models/
```

### 4. How to run es_qwen2?

#### 4.1 Adjust config.json

The es_qwen2 uses config.json to specify model directory, model name, path of qwen.tiktoken and path of embedding.bin.
The qwen.tiktoken is used for tokenization, the embedding.bin is used for generating embedding vector.
User can modify the config.json through own configurations. For example:

```json
"model_dir": "/opt/eswin/sample-code/npu_sample/qwen_sample/models/qwen2_0_5b_1k_int8/",
"block_models": [
    "modified_block_0_npu_b1.model",
    "modified_block_1_npu_b1.model",
    "modified_block_2_npu_b1.model",
    "modified_block_3_npu_b1.model",
    "modified_block_4_npu_b1.model",
    "modified_block_5_npu_b1.model",
    "modified_block_6_npu_b1.model",
    "modified_block_7_npu_b1.model",
    "modified_block_8_npu_b1.model",
    "modified_block_9_npu_b1.model",
    "modified_block_10_npu_b1.model",
    "modified_block_11_npu_b1.model",
    "modified_block_12_npu_b1.model",
    "modified_block_13_npu_b1.model",
    "modified_block_14_npu_b1.model",
    "modified_block_15_npu_b1.model",
    "modified_block_16_npu_b1.model",
    "modified_block_17_npu_b1.model",
    "modified_block_18_npu_b1.model",
    "modified_block_19_npu_b1.model",
    "modified_block_20_npu_b1.model",
    "modified_block_21_npu_b1.model",
    "modified_block_22_npu_b1.model",
    "modified_block_23_npu_b1.model"
],
"logits": true,
"logits_models": "lm_npu_b1.model",
"tokenizer_file": "/opt/eswin/sample-code/npu_sample/qwen_sample/src/qwen2_0_5b_1k_int8/qwen.tiktoken",
"embedding_file": "/opt/eswin/sample-code/npu_sample/qwen_sample/models/qwen2_0_5b_1k_int8/embedding.bin"
```

#### 4.2 Run es_qwen2

**run qwen2_0_5b**

```shell
cd /opt/eswin/sample-code/npu_sample/qwen_sample/bin/
./es_qwen2 /opt/eswin/sample-code/npu_sample/qwen_sample/src/qwen2_0_5b_1k_int8/config.json
```

**run qwen2_7b**

```shell
cd /opt/eswin/sample-code/npu_sample/qwen_sample/bin/
./es_qwen2 /opt/eswin/sample-code/npu_sample/qwen_sample/src/qwen2_7b_1k_int8_peer/config.json
```

#### 4.3 Debug tools

es_qwen2 support to enable debug functions through config.json. For example:

```json
"log_level": 0,
"data_dump": 1,
"data_dump_token": [10, 20],
"perf_enable": 1,
"op_dump": 1,
"op_dump_token": [2, 8],
"op_dump_layer": [1, 1],
"op_dump_type": 1,
"op_dump_list": [1, 10],
"log_level": The log level, 0: error, 1: message 2:debug
"data_dump": 1: Enbale dump token data, 0: Disable
"data_dump_token": The token range of data dump.
"perf_enable": 1: Enable print perfomance, 0: Disable.
"op_dump": 1: Enbale dump operation data, 0: Disbale.
"op_dump_token": The token range of operation dump.
"op_dump_layer": The layer range of operation dump.
"op_dump_type": Always set to 1.
"op_dump_list": The operation range of operation dump
```
